/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};
using namespace vcg;

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  int faceVisitCnt=0;
  int ccNum=0;
  for(MyMesh::FaceIterator fi=m.face.begin();fi!=m.face.end();++fi)
  {
    MyFace *fp = &*fi;
    if(!fp->IsV())
    {
      std::vector<MyFace *> unvisitedfaceStack;
      unvisitedfaceStack.push_back(fp);
      printf("Visiting connected component %i\n",ccNum);
      while(!unvisitedfaceStack.empty())
      {
        int ind=rand()%unvisitedfaceStack.size();
        std::swap(unvisitedfaceStack[ind],unvisitedfaceStack.back());
        fp=unvisitedfaceStack.back();
        unvisitedfaceStack.pop_back();
        if(!fp->IsV()) 
        {
          faceVisitCnt++;
          fp->C().SetGrayShade(float(faceVisitCnt)/float(m.fn));
          //printf("Visiting face %lu \n", tri::Index(m,fp));
          fp->SetV();
          for(int i=0;i<3;++i)
          {
            if(!fp->FFp(i)->IsV())
              unvisitedfaceStack.push_back(fp->FFp(i));
          }
        }
      }
      ++ccNum;
    }    
  }
   
  tri::io::ExporterOFF<MyMesh>::Save(m,"test.off",tri::io::Mask::IOM_FACECOLOR);
  
  return 0;
}






