/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};
using namespace vcg;

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;
  
  tri::Allocator<MyMesh>::AddVertex(m,Point3f( 1,  1 , 1));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f(-1, -1 , 1));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f(-1,  1, -1));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f( 1, -1, -1));
  
  MyVertex *vp0 = &m.vert[0];
  MyVertex *vp1 = &m.vert[1];
  MyVertex *vp2 = &m.vert[2];
  MyVertex *vp3 = &m.vert[3];
  
  tri::Allocator<MyMesh>::AddFace(m,vp0, vp2, vp1);
  tri::Allocator<MyMesh>::AddFace(m,vp0, vp1, vp3);
  tri::Allocator<MyMesh>::AddFace(m,vp0, vp3, vp2);
  tri::Allocator<MyMesh>::AddFace(m,vp1, vp2, vp3);
  
  
  
  
  tri::io::ExporterOFF<MyMesh>::Save(m,"test.off",tri::io::Mask::IOM_FACECOLOR);
  
  return 0;
}






